/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Dict
 *
 * The Initial Developer of the Original Code is David Costanzo
 * Portions created by the Initial Developer are Copyright (C) 2003-2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/////////////////////////////////////////////////////////////////////////////
// HACK: this block of code was pref.js. 
// For some reason we can't include it from context.xul
// 
const PREFNAME_DEFAULTSERVERNAME = "extensions.dict.defaultservername";
const PREFNAME_DEFAULTSERVERPORT = "extensions.dict.defaultserverport";

// Get the default server name, or use a default value
// Does not throw an exception.
function dictGetDefaultServerName() {

    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

    try {
         return pref.getCharPref(PREFNAME_DEFAULTSERVERNAME);
     }
     catch (e) {
     }

    // something is wrong with the preferences,
    // return a default value
    return 'dict.org';
}

// Get the default server port, or use a default value
// Does not throw an exception.
function dictGetDefaultServerPort() {

    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

    try {
         return pref.getCharPref(PREFNAME_DEFAULTSERVERPORT);
     }
     catch (e) {
     }

    try {
        // maybe it's stored as an int
        return pref.getIntPref(PREFNAME_DEFAULTSERVERPORT);
    }
    catch (e) {
    }

    // something is wrong with the preferences,
    // return a default value
    return '2628';
}
// END HACK
/////////////////////////////////////////////////////////////////////////////


// dictInit will be called every time a new browser window is created
window.addEventListener("load", dictInit, false);

// DictListener class, which is passed in as the "listener" parameter
// to DictClient::defineWord().
function DictListener () {
}

// wraps some HTML inside HTML/BODY tag and adds a title
DictListener.prototype = 
{

    createHtmlDocument : function (title, body)
    {
        var htmlDocument;
    
        htmlDocument = 
            '<HTML>\n'    +
            '  <HEAD>\n'  + 
            '      <TITLE>' + dictUtf8ToCharacterEntities(title) + '</TITLE>\n' +
            '  </HEAD>\n' +
            '  <BODY>\n'  +
            dictUtf8ToCharacterEntities(body) +
            '  </BODY>\n' +
            '</HTML>\n';
        
        return htmlDocument;
    },

    // Creates a new window and sets the document to be the given HTML.
    // Resizes the window to fit the content.
    createWindow : function (htmlDocument)
    {
        const features = [
            'status=no', 
            'toolbar=no',
            'menubar=no',
            'location=no',
            'width=450',
            'height=100'
        ];

        var popupWindow = window.open(
            '',
            '',
            features.join(','));
    
        popupWindow.document.clear();
        popupWindow.document.write(htmlDocument);
        popupWindow.document.close();
    
        // HACK: Figure out how to size the window
        //       to fit the content
        popupWindow.sizeToContent();
        popupWindow.innerHeight += 70;
        
        // If it's too large to display without scrollbars, shorten it
        if (600 < popupWindow.innerHeight) {
            // limit the height
            popupWindow.innerHeight = 600;
            
            // make wider to account for the width of the 
            // vertical scrollbar.  If we don't do this, a 
            // horizontal scrollbar will appear.
            popupWindow.innerWidth += 20;  
        }

        return popupWindow;
    },

    //
    // The next three functions are the DictListener interface
    //

    // Called when the DictClient has content to render
    // (either definitions or closest matches)
    onHaveDocument : function (title, body)
    {
        var htmlDocument = this.createHtmlDocument(title, body);
        
        this.createWindow(htmlDocument);
    },

    onNoDefinitionsFound : function (word) 
    {
        alert('No definitions were found for "' + word + '".\n');
    },
    
    onError : function (error)
    {
        alert(error);
    }
}


// Returns the selected text as a string.
function dictGetSelection() 
{
    var node = document.popupNode;

    var selection;

    var nodeLocalName = node.localName.toUpperCase();

    if ((nodeLocalName == "TEXTAREA") ||
        (nodeLocalName == "INPUT" && node.type == "text")) {

        // The selection is either in a TEXTAREA of an INPUT HTML node.
        // This includes the text area of Thunderbird (Mozilla Mail)
        selection = node.value.substring(
            node.selectionStart, 
            node.selectionEnd);
    }
    else {
        // The selection is in a web page
        var focusedWindow = document.commandDispatcher.focusedWindow;

        selection = focusedWindow.__proto__.getSelection.call(focusedWindow);
    }

    var selectedWord = selection.toString();

    // sanitize the selection
    selectedWord = selectedWord.replace(/^\W+/, ''); // trim leading non-alnum chars
    selectedWord = selectedWord.replace(/\W+$/, ''); // trim trailing non-alnum chars
    selectedWord = selectedWord.replace(/\s+/, ' '); // conflate whitespace runs

    return selectedWord;
}


function dictDefineSelection()
{
    var selectedWord = dictGetSelection();

    if (selectedWord.indexOf('"') != -1) {
        alert("Sorry, selections that contain quotation marks cannot be defined.");
        return;
    }

    if (selectedWord.length == 0) {
        alert("Please select a word to define.");
        return;
    }

    try {
        // read the host and port from the preferences
        var host = dictGetDefaultServerName();
        var port = dictGetDefaultServerPort();

        var listener = new DictListener();
        var client   = new DictClient();
        client.defineWord(host, port, selectedWord, listener);
    }
    catch (ex) {
        alert(ex);
    }
}


// Initialize the context menu in the calling XUL code.
function dictInit() {

    var contextMenu;

    // if this file is included from context.xul
    contextMenu = document.getElementById("contentAreaContextMenu");
    if (contextMenu != null) {
        contextMenu.addEventListener(
            "popupshowing",
            onDictPopup,
            false);  
    }

    // if this file is included from message-overlay.xul
    contextMenu = document.getElementById("messagePaneContext");
    if (contextMenu != null) {
        contextMenu.addEventListener(
            "popupshowing",
            onDictPopup,
            false);  
    }
}



// hide the Dictionary Search item when apropriate (use same logic as for web search)
function onDictPopup() {

    var item = document.getElementById('dict-define-selection');

    var selection = dictGetSelection();
    if (selection != null && selection.length != 0) {

        const LENGTHCUTOFF = 15;

        if (LENGTHCUTOFF < selection.length) {
            // selection is too long to appear in menu.  It must be abbreviated.
            selection = selection.slice(0, LENGTHCUTOFF) + '...';
        }

        var dictStringBundle = document.getElementById("dict_string_bundle");
        var labelTemplate = dictStringBundle.getString("context_searchselect.label");

        item.setAttribute("label", labelTemplate.replace('%S', selection));
        item.hidden = false; // show the menu item
    }
    else {
        item.hidden = true; // hide the menu item
    }
}
